@@ -1,11 +1,14 @@
-import typer, requests, os, zipfile, glob, shutil, json, sys
+import typer, requests, os, zipfile, glob, shutil, json, sys, time, logging
 from pathlib import Path
 import snow_pyrepl as pyrepl
 from typing import Optional
 from replit.database import Database
 import getpass
+from watchdog.observers import Observer
+from watchdog.events import LoggingEventHandler
 
-__version__ = "1.1.6"
+
+__version__ = "1.2.0"
 homedir = Path.home()
 homedir = str(homedir).replace("\\", "/")
 try:
@@ -16,7 +19,18 @@ except:
 app = typer.Typer()
 
 def get_json(user, replname, sid):
-	return requests.get(f"https://replit.com/data/repls/@{user}/{replname}", cookies={"connect.sid": sid}).json()
+	headers = {
+		'Content-Type': 'application/json',
+		'connect.sid': sid,
+		'X-Requested-With': 'https://replit.com',
+		'Origin': 'https://replit.com',
+		'User-Agent': 'Mozilla/5.0'
+	}
+	cookies = {
+		"connect.sid": sid
+	}
+	r = requests.get(f"https://replit.com/data/repls/@{user}/{replname}", headers=headers, cookies=cookies)
+	return r.json()['id']
 
 @app.callback()
 def callback(ctx: typer.Context):
@@ -131,6 +145,80 @@ def pull(override:bool=False):
 		shutil.rmtree(".temp")
 	typer.echo(f"Refreshed Local Repl Dir")
 
+
+@app.command(help="Create a live file watching service to automatically save changes")
+def livewatch():
+	f = open(".replitcliconfig", "r")
+	content = f.read()
+	f.close()
+	url = content.split("=")[1].split("\n")[0]
+	uuid = content.split("=")[-1].strip()
+	sid = __sid__.strip()
+	slug = url.split("/@")[-1].split("\n")[0].strip()
+	user, replname = slug.split("/")[0], slug.split("/")[1]
+	data = requests.get(f"https://replit.com/data/repls/@{slug}", cookies={"connect.sid": sid}).json()
+	if not data['is_owner']:
+		typer.echo("You do not have the correct permissions to write to this repl.")
+		return
+
+	key = __sid__.strip()
+
+	def delete_file(filename):
+		r = requests.delete("https://replops.coolcodersj.repl.co", data=json.dumps({
+			"UUID": uuid,
+			"username": user,
+			"repl": replname,
+			"sid": key,
+			"filepath": filename
+		}),
+		headers = {'Content-type': 'application/json', 'Accept': 'text/plain'})
+
+	def write_file(filename, content):
+		r = requests.delete("https://replops.coolcodersj.repl.co", data=json.dumps({
+			"UUID": uuid,
+			"username": user,
+			"repl": replname,
+			"sid": key,
+			"filepath": filename,
+			"content": content
+		}),
+		headers = {'Content-type': 'application/json', 'Accept': 'text/plain'})
+
+	class Event(LoggingEventHandler):
+		def dispatch(self, event):
+			path = event.src_path.replace("\\", "/")[2:]
+			if str(type(event)) == "<class 'watchdog.events.FileCreatedEvent'>":
+				print(f"Created {path}")
+				write_file(path, "")
+			if str(type(event)) == "<class 'watchdog.events.FileModifiedEvent'>":
+				print(f"Modified {path}")
+				write_file(path, open(path, "r").read())
+			if str(type(event)) == "<class 'watchdog.events.FileMovedEvent'>":
+				destPath = event.dest_path.replace("\\\\", "/")[2:]
+				print(f"Moved {path} to {destPath}")
+				delete_file(path)
+				write_file(destPath, open(destPath, "r").read())
+			if str(type(event)) == "<class 'watchdog.events.FileDeletedEvent'>":
+				print(f"Deleted {path}")
+				delete_file(path)
+
+	logging.basicConfig(level=logging.INFO,
+						format='%(asctime)s - %(message)s',
+						datefmt='%Y-%m-%d %H:%M:%S')
+	path = "."
+	event_handler = Event()
+	observer = Observer()
+	observer.schedule(event_handler, path, recursive=True)
+	observer.start()
+	print("File watching service started....enter ^C to quit. (CTRL + C or CMD + C)")
+	try:
+		while True:
+			time.sleep(1)
+	except KeyboardInterrupt:
+		observer.stop()
+	observer.join()
+
+
 @app.command(help="Push changes to the server and override remote.")
 def push(override:bool=False):
 	f = open(".replitcliconfig", "r")
@@ -140,43 +228,52 @@ def push(override:bool=False):
 	uuid = content.split("=")[-1].strip()
 	sid = __sid__.strip()
 	slug = url.split("/@")[-1].split("\n")[0].strip()
+	user, replname = slug.split("/")[0], slug.split("/")[1]
 	data = requests.get(f"https://replit.com/data/repls/@{slug}", cookies={"connect.sid": sid}).json()
 	if not data['is_owner']:
 		typer.echo("You do not have the correct permissions to write to this repl.")
 		return
 
+	key = __sid__.strip()
+
 	if not override:
 		files = glob.glob("*")
 		files2 = glob.glob(".*")
 		for file in files2:
 			files.append(file)
 		done = []
+		noExtFile = False
 		for file in files:
 			print(f"FILE/DIR FOUND - {file}")
 			if "connect.sid" not in file and ".replitcliconfig" not in file:
 				newfile = file
 				if "." not in file:
 					newfile = file + "/"
-
-				if "." not in newfile:
 					filelist = glob.glob(f"{newfile}*")
-					typer.echo("Found Sub-Files/Dirs")
-					for file in filelist:
-						files.append(file.replace("\\", "/"))
-						file = file.replace("\\", "/")
-						typer.echo(f"Appending file {file} to list.")
+					if filelist == []:
+						noExtFile = True
+						newfile = file
+					else:
+						typer.echo("Found Sub-Files/Dirs")
+						for file in filelist:
+							files.append(file.replace("\\", "/"))
+							file = file.replace("\\", "/")
+							typer.echo(f"Appending file {file} to list.")
 
-				if "." in newfile:
+				if "." in newfile or noExtFile:
 					print(f"REFRESHING FILE/DIR ON SERVER - {newfile}")
-					requests.post("https://replops.coolcodersj.repl.co", data=json.dumps({
-					"UUID": uuid,
-					"SID": sid,
-					"filepath": newfile,
-					"content": open(newfile, "rb").read().decode("utf-8")
+					r = requests.post("https://replops.coolcodersj.repl.co", data=json.dumps({
+						"UUID": uuid,
+						"username": user,
+						"repl": replname,
+						"sid": key,
+						"filepath": newfile,
+						"content": open(newfile, "r").read()
 					}),
 					headers = {'Content-type': 'application/json', 'Accept': 'text/plain'})
 
 				done.append(newfile)
+				noExtFile = False
 		typer.echo("Remote Repl Refreshed Successfully")
 	else:
 		zip = requests.get(f"{url}.zip", cookies={"connect.sid": sid})
@@ -191,28 +288,34 @@ def push(override:bool=False):
 		for file in files2:
 			files.append(file.replace("\\", "/"))
 		done = []
+		noExtFile = False
 		for file in files:
 			file = file.replace("\\", "/")
 			if "connect.sid" not in file and ".replitcliconfig" not in file:
 				newfile = file
 				if "." not in file:
 					newfile = file + "/"
-
-				if "." not in newfile:
 					filelist = glob.glob(f"{newfile}*")
-					for file in filelist:
-						files.append(file.replace("\\", "/"))
-						file = file.replace("\\", "/")
+					if filelist == []:
+						noExtFile = True
+						newfile = file
+					else:
+						for file in filelist:
+							files.append(file.replace("\\", "/"))
+							file = file.replace("\\", "/")
 
-				if "." in newfile:
-					requests.delete("https://replops.coolcodersj.repl.co", data=json.dumps({
-					"UUID": uuid,
-					"SID": sid,
-					"filepath": newfile.split(".temp/")[-1],
+				if "." in newfile or noExtFile:
+					r = requests.delete("https://replops.coolcodersj.repl.co", data=json.dumps({
+						"UUID": uuid,
+						"username": user,
+						"repl": replname,
+						"sid": key,
+						"filepath": newfile,
 					}),
 					headers = {'Content-type': 'application/json', 'Accept': 'text/plain'})
 
 				done.append(newfile)
+				noExtFile = False
 		shutil.rmtree(".temp")
 		files = glob.glob("*")
 		files2 = glob.glob(".*")
@@ -236,11 +339,13 @@ def push(override:bool=False):
 
 				if "." in newfile:
 					print(f"REFRESHING FILE/DIR ON SERVER - {newfile}")
-					requests.post("https://replops.coolcodersj.repl.co", data=json.dumps({
-					"UUID": uuid,
-					"SID": sid,
-					"filepath": newfile,
-					"content": open(newfile, "rb").read().decode("utf-8")
+					r = requests.post("https://replops.coolcodersj.repl.co", data=json.dumps({
+						"UUID": uuid,
+						"username": user,
+						"repl": replname,
+						"sid": key,
+						"filepath": newfile,
+						"content": open(newfile, "r").read()
 					}),
 					headers = {'Content-type': 'application/json', 'Accept': 'text/plain'})
 
@@ -251,16 +356,16 @@ def login(sid:str=None):
 	if sid != None:
 		if not os.path.exists(f"{homedir}/replit-cli"):
 			os.mkdir(f"{homedir}/replit-cli")
-			f = open(f"{homedir}/replit-cli/connect.sid", "w")
-			print(f"""{sid}""", file=f)
-			f.close()
-			typer.echo(f"Your SID value has been set as {sid}")
-			return
+		f = open(f"{homedir}/replit-cli/connect.sid", "w")
+		print(f"""{sid}""", file=f)
+		f.close()
+		typer.echo(f"Your SID value has been set as {sid}")
+		return
 	username = typer.prompt("Enter your username")
 	password = getpass.getpass()
 	openconfirm = typer.confirm("Replit Login requires an hcaptcha token. You can retrieve one from https://sjurl.tk/captcha. Would you like to visit this site?")
 	if not openconfirm:
-		typer.echo("Replit Login has been aborted since you denied access to open the hcaptcha site.")		
+		typer.echo("Replit Login has been aborted since you denied access to open the hcaptcha site.")
 		return
 
 	typer.launch("https://sjurl.tk/captcha")
@@ -318,9 +423,9 @@ def run(repl:str, run:bool=True, stop:bool=False, restart:bool=False):
 	user = repl.split("/")[0]
 	replname = repl.split("/")[1]
 
-	id = get_json(user, replname, sid)['id']
 	key = __sid__.strip()
-	token, url = pyrepl.get_token(id, key)
+	token, url = pyrepl.get_token(user, replname, key)
+	id = get_json(user, replname, key)
 	client = pyrepl.Client(token, id, url)
 	channel = client.open('shellrun2', 'runner')
 	if run:
@@ -379,8 +484,8 @@ def shell(repl:str):
 	replname = repl.split("/")[1]
 
 	key = __sid__.strip()
-	id = get_json(user, replname, key)['id']
-	token, url = pyrepl.get_token(id, key)
+	token, url = pyrepl.get_token(user, replname, key)
+	id = get_json(user, replname, key)
 	client = pyrepl.Client(token, id, url)
 	channel = client.open('exec', 'runner')
 	while True:
@@ -418,8 +523,8 @@ def exec(repl:str, cmd:str):
 	replname = repl.split("/")[1]
 
 	key = __sid__.strip()
-	id = get_json(user, replname, key)['id']
-	token, url = pyrepl.get_token(id, key)
+	token, url = pyrepl.get_token(user, replname, key)
+	id = get_json(user, replname, key)
 	client = pyrepl.Client(token, id, url)
 	channel = client.open('exec', 'runner')
 	output = channel.get_output({
@@ -469,6 +574,15 @@ def env(contents:bool=True, key:str="", value:str="", delete:str=""):
 	r = requests.get(f"https://replit.com/data/repls/@{slug}", cookies={"connect.sid": sid}).json()
 	id = r['id']
 
+	user, replname = slug.split("/")[0], slug.split("/")[1]
+	data = requests.get(f"https://replit.com/data/repls/@{slug}", cookies={"connect.sid": sid}).json()
+	if not data['is_owner']:
+		typer.echo("You do not have the correct permissions to write to this repl.")
+		return
+
+	key = __sid__.strip()
+
+
 	if not os.path.exists(f".tempcache/{file}"):
 		f = open(f".tempcache/{file}", "x")
 		f.close()
@@ -505,18 +619,16 @@ def env(contents:bool=True, key:str="", value:str="", delete:str=""):
 	f = open(".env", "w")
 	f.write(string)
 	f.close()
-	requests.post("https://replops.coolcodersj.repl.co", data=json.dumps({
-		"UUID": str(id.strip()),
-		"SID": sid,
+
+	r = requests.post("https://replops.coolcodersj.repl.co", data=json.dumps({
+		"UUID": id,
+		"username": user,
+		"repl": replname,
+		"sid": key,
 		"filepath": file,
 		"content": string
-		}
-	),
-	headers = {
-	'Content-type': 'application/json',
-	'Accept': 'text/plain'
-	}
-	)
+	}),
+	headers = {'Content-type': 'application/json', 'Accept': 'text/plain'})
 
 @app.command(help="Edit the Replit DB for a Repl")
 def db(url:str, data:bool=False, key:str="", value:str="", delete:str=""):
